Option Explicit On

Imports System.Net
Imports System.Net.Sockets
Imports System.Text.Encoding
Imports System.Threading

Public Class clsTcpServer

    Implements IDisposable

    Public Clients As New Collection
    Public MaxClients As Integer = 0

    Private thisSock As Socket
    Private thisListener As TcpListener
    Private thisThread As Thread
    Private callbackNewClientSub As TcpServerNewClientSub
    Private callbackDisconnectSub As TcpServerClientDisconnectSub

    Public Delegate Sub TcpServerNewClientSub(ByVal newClient As clsTcpClient)
    Public Delegate Sub TcpServerClientDisconnectSub(ByVal thisClient As clsTcpClient)

    Public Const DefaultMaxClients As Integer = 0 ' no limit

    Private Const SOCKET_RECV_TIMEOUT = 2000 ' 2 seconds
    Private Const SOCKET_SEND_TIMEOUT = 2000 ' 2 seconds
    Private Const LOOP_SLEEP_DELAY = 10 ' 1/100 second

    Public Sub New(ByVal ip As String, ByVal port As Integer, _
        Optional ByVal max_clients As Integer = DefaultMaxClients, _
        Optional ByVal clientDisconnectCallback As TcpServerClientDisconnectSub = Nothing)

        Me.MaxClients = max_clients
        Me.callbackDisconnectSub = clientDisconnectCallback

        ' instantiate a new tcp socket
        thisSock = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)

        ' configure the socket
        thisSock.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReceiveTimeout, SOCKET_RECV_TIMEOUT)
        thisSock.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.SendTimeout, SOCKET_SEND_TIMEOUT)

        thisListener = New TcpListener(IPAddress.Parse(ip), port)

    End Sub

    Public ReadOnly Property Ready() As Boolean
        Get
            Return Not (thisSock Is Nothing)
        End Get
    End Property

    Public ReadOnly Property IsListening() As Boolean
        Get
            Return Not (thisThread Is Nothing)
        End Get
    End Property

    Public Sub StartListener(ByVal newClientCallback As TcpServerNewClientSub)

        If thisThread Is Nothing Then

            callbackNewClientSub = newClientCallback
            thisListener.Start()

            thisThread = New Thread(AddressOf _threadedListener)
            thisThread.Priority = ThreadPriority.BelowNormal
            thisThread.IsBackground = True
            thisThread.Start()

        End If

    End Sub

    Public Sub StopListener()

        If Not (thisThread Is Nothing) Then

            thisThread.Abort()
            thisThread = Nothing

            thisListener.Stop()

        End If

    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose

        Do While Clients.Count > 0

            Dim thisClient As clsTcpClient = Clients.Item(Clients.Count)
            Clients.Remove(thisClient.ToString)
            thisClient.Dispose()

        Loop

        StopListener()
        thisSock.Close()

    End Sub

    Public Sub _threadedListener()

        Do Until Not IsListening

            ' check for max connections
            If DefaultMaxClients = 0 Or _
                Clients.Count < MaxClients Then

                ' check for pending connection
                If thisListener.Pending() Then

                    Try

                        ' accept the incoming connection on a new thread
                        With New Thread(AddressOf _threadedAccept)
                            .IsBackground = True
                            .Priority = ThreadPriority.AboveNormal
                            .Start()
                        End With

                    Catch
                    End Try

                End If

            End If

            ' sleep for a duration of time
            Thread.Sleep(LOOP_SLEEP_DELAY)

        Loop

    End Sub

    Private Sub _threadedAccept()

        Try

            Dim newClient As clsTcpClient = Nothing

            ' dummy object used for sync locking
            Dim objLock As New Object

            ' lock the object to this thread
            SyncLock objLock

                ' accept the incoming connection
                newClient = New clsTcpClient(thisListener.AcceptSocket(), _
                    New clsTcpClient.TcpClientDisconnectSub(AddressOf _clientDisconnectCallback))

            End SyncLock

            objLock = Nothing

            ' validate the new connection
            If Not (newClient Is Nothing) Then

                ' add the new client to the collection
                Clients.Add(newClient, newClient.ToString)

                ' raise new client callback func
                If Not (callbackNewClientSub Is Nothing) Then _
                    callbackNewClientSub.Invoke(newClient)

            End If

        Catch
        End Try

    End Sub

    Private Sub _clientDisconnectCallback(ByVal thisClient As clsTcpClient)

        ' remove the disconnected client
        Clients.Remove(thisClient.ToString)

        ' raise client disconnect callback
        If Not (Me.callbackDisconnectSub Is Nothing) Then _
            Me.callbackDisconnectSub.Invoke(thisClient)

    End Sub

End Class
